'-----------------------------------------------------------------------------
'--- File: export.excel.template.vbs
'--- Description: This VBScript example exports the MS Excel spreadsheet cells
'---   "A1:D20" from "Sheet1" in "c:\myexisting.xls" to the SAS data set
'---   "sasuser.mydata" using The IOM Provider.  You will have to change the
'---   values in the "Configuration vars" section to identify your dataset,
'---   connection, xls file, etc.  Nothing else should need to be changed.
'--- Note:  This script depends on the first row of the selection having a 
'---   particular format that indicates the name, type, and width of each
'---   column.  That format is "name:type:width".  For example, a column named
'---   Col1 that is numeric and has a width of 4 must have "Col1:num:4" in the
'---   first cell.  Character columns are specified as type "chr".  Other
'---   methods to obtain column information are beyond the scope of this
'---   example.
'-----------------------------------------------------------------------------

Option Explicit

'-----------------------------------------------------------------------------
'--- Constants

const ERR_NOBOOK = -1
const ERR_NOSHEET = -2
const ERR_NORECORDSET = -3
const ERR_NORANGE = -4
const adOpenForwardOnly = 0
const adOpenDynamic = 2
const adLockReadOnly = 1
const adLockOptimistic = 3
const adCmdTableDirect = 512
const adCmdText = 1
const xlThin = 2
const xlEdgeBottom = 9

'-----------------------------------------------------------------------------
'--- Configuration vars
Dim Filename, SheetName, Selection, SASConnectionString, DatasetName, DatasetType

'--- Full path to the .xls file
Filename = "C:\myexisting.xls"

'--- Name of the sheet to export
SheetName = "Sheet1"

'--- Name of the selection to export
Selection = "A1:D20"

'--- Connection string for SAS OLE DB provider
SASConnectionString = "Provider=SAS.IOMProvider.1;Data Source=_LOCAL_" 

'--- libref.member
DatasetName = "sasuser.mydata"

'--- DatasetType should be adCmdTableDirect.
DatasetType = adCmdTableDirect

'-----------------------------------------------------------------------------
'--- GetBook: Returns the opened workbook specified by Filename.
'---   Prompts the user if Filename is the empty string.

Private Function GetBook( appXL )
  Set GetBook = appXL.Workbooks.Open( Filename )
End Function

'-----------------------------------------------------------------------------
'--- GetSheet: Creates a sheet named DatasetName + " data" in the workbook
'---   Book.

Private Function GetSheet( Book )
  Dim res
  On Error Resume Next
  Set GetSheet = Nothing
  Set GetSheet = Book.Sheets( SheetName )
  On Error GoTo 0
  If GetSheet Is Nothing Then
    MsgBox SheetName & " worksheet not found.", vbOK Or vbExclamation, _
           "SAS Export Data Wizard"
  End If
End Function

'-----------------------------------------------------------------------------
'--- GetSheet: Returns a range object for the cells in Sheet specified by
'---   Selection.

Private Function GetRange( Sheet )
  Set GetRange = Sheet.Range(Selection)
End Function

'-----------------------------------------------------------------------------
'--- GetSASRecordset: Creates and returns an ADO Recordset on the dataset
'---   specified by DatasetName.  The structure of the new data set is
'---   determined by the cells in Rng.  The types of the columns are returned
'---   in the Types parameter as an array of strings.

Private Function GetSASRecordset( Rng, Types )
  Dim res, i, j, bFirst, elm, typename, colname, width, pos, sql
  Dim Connection
  ReDim Types( Rng.Columns.Count - 1 )
  Set Connection = WScript.CreateObject("ADODB.Connection")
  Set GetSASRecordset = WScript.CreateObject("ADODB.Recordset")

  Connection.Open SASConnectionString
  On Error Resume Next
  GetSASRecordset.Open DatasetName, Connection, adOpenForwardOnly, _
                       adLockReadOnly, adCmdTableDirect
  If Err.Number = 0 Then
    GetSASRecordset.Close
    res = MsgBox( DatasetName & " table found." & vbNewLine & _
                  "Click ""Ok"" to continue and overwrite this table.", _
                  vbOKCancel Or vbExclamation, "SAS Export Data Wizard" )
    If res = vbCancel Then
      Set GetSASRecordset = Nothing
      Exit Function
    End If
  ElseIf Err.Number <> -2147217865 Then
    MsgBox "Unexpected error: " & Err.Description & ": " & Err.Number
    Set GetSASRecordset = Nothing
    Exit Function
  End If
  On Error GoTo 0
  sql = "create table " & DatasetName & " ("
  bFirst = True
  i = 1
  j = 1
  For j = 1 to Rng.Columns.Count
    Set elm = Rng.Cells(i, j)
    If IsEmpty(elm) Then Exit For
    typename = ""
    colname = elm.Value
    width = "8"
    pos = InStr(elm.Value, ":")
    If pos <> 0 Then
      typename = Mid(elm.Value, pos + 1)
      colname = Left(elm.Value, pos - 1)
      pos = InStr(typename, ":")
      If pos <> 0 Then
        width = Mid(typename, pos + 1)
        typename = Left(typename, pos - 1)
      End If
    End If
    Types(j - 1) = typename
    If Not bFirst Then sql = sql & ", "
    bFirst = False
    If typename = "chr" Then
      sql = sql & colname & " char(" & width & ")"
    ElseIf typename = "num" Or typename = "" Then
      sql = sql & colname & " num(" & width & ")"
    End If
  Next
  sql = sql & ")"
  Connection.Execute sql
    
  GetSASRecordset.Open DatasetName, Connection, adOpenDynamic, _
                       adLockOptimistic, adCmdTableDirect
End Function

'-----------------------------------------------------------------------------
'--- CopyData: Copies all data from Rng (an open Excel Range object) to
'--- Recordset (an open ADO Recordset object)

Private Sub CopyData( Rng, Types, Recordset )
  Dim i, j, elm, vt

  For i = 2 To Rng.Rows.Count
    Recordset.AddNew
    For j = 1 To Rng.Columns.Count
      Set elm = Rng.Cells(i, j)
      If Not IsEmpty(elm) Then
        vt = VarType(elm.Value)
        If vt = 8 Then ' elm.Value is character data
          If Types(j - 1) = "chr" Then ' column is character data
            Recordset.Fields(j - 1) = elm.Value
          Else ' column is numeric data
            Recordset.Fields(j - 1) = Null ' CDbl(elm.Value)
          End If
        ElseIf vt = 5 Then ' elm.Value is numeric data
          If Types(j - 1) = "chr" Then ' column is character data
            Recordset.Fields(j - 1) = CStr(elm.Value)
          Else ' column is numeric data
            Recordset.Fields(j - 1) = elm.Value
          End If
        End If
      End If
    Next
    Recordset.Update
  Next
End Sub

'-----------------------------------------------------------------------------
'--- Main: The main controling routine.  Creates a Recordset object on an
'---   new SAS dataset and a Sheet object on an existing Excel spreadsheet
'---   and then copies the cell data into the SAS dataset.

Private Sub Main()
  Dim appXL
  Dim Book, Sheet, Rng, Types
  Dim Recordset

  Set appXL = WScript.CreateObject("Excel.Application")

  Set Book = GetBook( appXL )
  if Book Is Nothing Then WScript.Quit( ERR_NOBOOK )

  Set Sheet = GetSheet( Book )
  If Sheet Is Nothing Then
    Book.Close false
    WScript.Quit( ERR_NOSHEET )
  End If

  Set Rng = GetRange( Sheet )
  If Rng Is Nothing Then
    Book.Close false
    WScript.QUIT( ERR_NORANGE )
  End If

  Set Recordset = GetSASRecordset( Rng, Types )
  If Recordset Is Nothing Then
    Book.Close false
    WScript.Quit( ERR_NORECORDSET )
  End If

  CopyData Rng, Types, Recordset

  Recordset.Close
  Book.Close false
End Sub

'-----------------------------------------------------------------------------
'--- Start the Main Sub

Main

